@@ -6,7 +6,7 @@ Kafka::Exceptions - Perl Kafka API exception definitions.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Exceptions> version 1.001013 .
+This documentation refers to C<Kafka::Exceptions> version 1.02 .
 
 =cut
 
@@ -20,7 +20,7 @@ use warnings;
 
 our $DEBUG = 0;
 
-our $VERSION = '1.001013';
+our $VERSION = '1.02';
 
 use Exporter qw(
     import
@@ -37,7 +37,7 @@ use Exception::Class (
     },
     'Kafka::Exception::Connection' => {
         isa     => 'Kafka::Exception',
-        fields  => [ 'request', 'response' ],
+        fields  => [ 'request', 'response', 'io_error' ],
     },
     'Kafka::Exception::Consumer' => {
         isa     => 'Kafka::Exception',
@@ -291,7 +291,7 @@ Vlad Marchenko
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2016 by TrackingSoft LLC.
+Copyright (C) 2012-2017 by TrackingSoft LLC.
 
 This package is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself. See I<perlartistic> at
