@@ -6,7 +6,7 @@ Kafka::IO - Interface to network communication with the Apache Kafka server.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::IO> version 1.001013 .
+This documentation refers to C<Kafka::IO> version 1.02 .
 
 =cut
 
@@ -20,7 +20,7 @@ use warnings;
 
 our $DEBUG = 0;
 
-our $VERSION = '1.001013';
+our $VERSION = '1.02';
 
 #-- load the modules -----------------------------------------------------------
 
@@ -245,13 +245,16 @@ sub new {
 
     $self->{socket}     = undef;
     $self->{_io_select} = undef;
+    my $error;
     try {
         $self->_connect();
     } catch {
-        my $error = $_;
-        $self->_error( $ERROR_CANNOT_BIND, format_message("Kafka::IO(%s:%s)->new: %s", $self->{host}, $self->{port}, $error ) );
+        $error = $_;
     };
 
+    $self->_error( $ERROR_CANNOT_BIND, format_message("Kafka::IO(%s:%s)->new: %s", $self->{host}, $self->{port}, $error ) )
+        if defined $error
+    ;
     return $self;
 }
 
@@ -446,6 +449,7 @@ sub receive {
             if ( defined( $from_recv ) && length( $buf ) ) {
                 $message .= $buf;
                 $len_to_read = $length - length( $message );
+                --$retries; # this attempt was successful, don't count as a retry
             }
             if ( $errno ) {
                 if ( $errno == EINTR ) {
@@ -862,7 +866,10 @@ sub _debug_msg {
 # Handler for errors
 sub _error {
     my $self = shift;
-    Kafka::Exception::IO->throw( throw_args( @_ ) );
+    my %args = throw_args( @_ );
+    $self->_debug_msg( format_message( 'throwing IO error %s: %s', $args{code}, $args{message} ) )
+        if $self->debug_level;
+    Kafka::Exception::IO->throw( %args );
 }
 
 #-- Closes and cleans up -------------------------------------------------------
@@ -873,7 +880,7 @@ __END__
 
 =head1 DIAGNOSTICS
 
-When error is detected, an exception, represented by object of C<Kafka::Exception::Producer> class,
+When error is detected, an exception, represented by object of C<Kafka::Exception::IO> class,
 is thrown (see L<Kafka::Exceptions|Kafka::Exceptions>).
 
 L<code|Kafka::Exceptions/code> and a more descriptive L<message|Kafka::Exceptions/message> provide
@@ -983,9 +990,11 @@ Sergiy Zuban
 
 Vlad Marchenko
 
+Damien Krotkine
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2016 by TrackingSoft LLC.
+Copyright (C) 2012-2017 by TrackingSoft LLC.
 
 This package is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself. See I<perlartistic> at
