@@ -6,7 +6,7 @@ Kafka::Protocol - Functions to process messages in the Apache Kafka protocol.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Protocol> version 1.001013 .
+This documentation refers to C<Kafka::Protocol> version 1.02 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '1.001013';
+our $VERSION = '1.02';
 
 use Exporter qw(
     import
@@ -32,6 +32,8 @@ our @EXPORT_OK = qw(
     encode_metadata_request
     encode_offset_request
     encode_produce_request
+    encode_api_versions_request
+    decode_api_versions_response
     _decode_MessageSet_template
     _decode_MessageSet_array
     _encode_MessageSet_array
@@ -39,7 +41,8 @@ our @EXPORT_OK = qw(
     _pack64
     _unpack64
     _verify_string
-    $APIVERSION
+    $DEFAULT_APIVERSION
+    $IMPLEMENTED_APIVERSIONS
     $BAD_OFFSET
     $COMPRESSION_CODEC_MASK
     $CONSUMERS_REPLICAID
@@ -51,14 +54,7 @@ our @EXPORT_OK = qw(
 
 use Compress::Snappy;
 use Const::Fast;
-use IO::Compress::Gzip qw(
-    gzip
-    $GzipError
-);
-use IO::Uncompress::Gunzip qw(
-    gunzip
-    $GunzipError
-);
+use Gzip::Faster qw( gzip gunzip );
 use Params::Util qw(
     _ARRAY
     _HASH
@@ -69,6 +65,7 @@ use Scalar::Util qw(
     dualvar
 );
 use String::CRC32;
+use Try::Tiny;
 
 use Kafka qw(
     $BITS64
@@ -83,8 +80,6 @@ use Kafka qw(
     $ERROR_NOT_BINARY_STRING
     $ERROR_REQUEST_OR_RESPONSE
     $NOT_SEND_ANY_RESPONSE
-    $RECEIVE_EARLIEST_OFFSETS
-    $RECEIVE_LATEST_OFFSET
     $WAIT_WRITTEN_TO_LOCAL_LOG
 );
 use Kafka::Exceptions;
@@ -93,6 +88,7 @@ use Kafka::Internals qw(
     $APIKEY_METADATA
     $APIKEY_OFFSET
     $APIKEY_PRODUCE
+    $APIKEY_APIVERSIONS
     $PRODUCER_ANY_OFFSET
     format_message
 );
@@ -346,13 +342,21 @@ The following constants are available for export
 
 =cut
 
-=head3 C<$APIVERSION>
+=head3 C<$DEFAULT_APIVERSION>
 
-According to Apache Kafka documentation: 'This is a numeric version number for this api.
-Currently the supported version for all APIs is 0 .'
+The default API version that will be used as fallback, if it's not possible to
+detect what the Kafka server supports. Only Kafka servers > 0.10.0.0 can be
+queried to get which API version they implements. On Kafka servers 0.8.x and
+0.9.x, the protocol will default to use $DEFAULT_APIVERSION. Currently its
+value is '0'
 
 =cut
-const our $APIVERSION                   => 0;
+
+const our $DEFAULT_APIVERSION                   => 0;
+
+# HashRef, keys are APIKEYs, values are the api version that this protocol
+# implements. If not populated, the connection will use the $DEFAULT_APIVERSION
+our $IMPLEMENTED_APIVERSIONS = {};
 
 # Attributes
 
@@ -398,7 +402,9 @@ my ( $_ProduceRequest_header_template,      $_ProduceRequest_header_length ) = (
     10
 );
 my ( $_MessageSet_template,                 $_MessageSet_length ) = (
-    q{a[8]l>},              #    a8                  # 8 Offset
+    # qq{${_int64_template}l>},
+    q{a[8]l>},
+                            # 8 Offset
                             # 4 MessageSize
     12
 );
@@ -410,8 +416,19 @@ my ( $_FetchRequest_header_template,        $_FetchRequest_header_length ) = (
                             # 4 topics array size
     16
 );
+my ( $_FetchRequest_header_template_v3,     $_FetchRequest_header_length_v3 ) = (
+    q{l>l>l>l>l>},
+                            # 4 ReplicaId
+                            # 4 MaxWaitTime
+                            # 4 MinBytes
+                            # 4 MaxBytes
+                            # 4 topics array size
+    20
+);
 my ( $_FetchRequest_body_template,          $_FetchRequest_body_length ) = (
-    q{l>a[8]l>},            # 4 Partition
+#    qq{l>${_int64_template}l>},
+    q{l>a[8]l>},
+                            # 4 Partition
                             # 8 FetchOffset
                             # 4 MaxBytes
     16
@@ -422,17 +439,33 @@ my ( $_OffsetRequest_header_template,       $_OffsetRequest_header_length ) = (
     8
 );
 my ( $_OffsetRequest_body_template,         $_OffsetRequest_body_length ) = (
-    q{l>a[8]l>},            # 4 Partition
+#    qq{l>${_int64_template}l>},
+    q{l>a[8]l>},
+                            # 4 Partition
                             # 8 Time
                             # 4 MaxNumberOfOffsets
     16
 );
+my ( $_OffsetRequest_body_template_v1,      $_OffsetRequest_body_length_v1 ) = (
+#    qq{l>${_int64_template}},
+    q{l>a[8]},
+                          # 4 Partition
+                          # 8 Time
+    12
+);
 my ( $_FetchResponse_header_template,       $_FetchResponse_header_length ) = (
     q{x[l]l>l>},            # Size (skip)
                             # 4 CorrelationId
                             # 4 topics array size
     8
 );
+my ( $_FetchResponse_header_template_v1,       $_FetchResponse_header_length_v1 ) = (
+    q{x[l]l>l>l>},          # Size (skip)
+                            # 4 CorrelationId
+                            # 4 throttle_time_ms
+                            # 4 topics array size
+    12
+);
 my ( $_Message_template,                    $_Message_length ) = (
     qq(${_int64_template}l>l>ccl>),
                             # 8 Offset
@@ -443,6 +476,18 @@ my ( $_Message_template,                    $_Message_length ) = (
                             # Key length
     8                       # Only Offset length
 );
+my ( $_Message_template_with_timestamp,     $_Message_length_with_timestamp ) = (
+    qq(${_int64_template}l>l>cc${_int64_template}l>),
+                            # 8 Offset
+                            # MessageSize
+                            # Crc
+                            # MagicByte
+                            # Attributes
+                            # Timestamp
+                            # Key length
+    8                       # Only Offset length
+);
+
 my ( $_FetchResponse_topic_body_template,   $_FetchResponse_topic_body_length )= (
     qq(s>/al>l>s>${_int64_template}),
                             # TopicName
@@ -462,6 +507,101 @@ The following functions are available for C<Kafka::MockProtocol> module.
 
 =cut
 
+=head3 C<encode_api_versions_request( $ApiVersions_Request )>
+
+Encodes the argument and returns a reference to the encoded binary string
+representing a Request buffer.
+
+This function take arguments. The following argument is currently recognized:
+
+=over 3
+
+=item C<$ApiVersions_Request>
+
+C<$ApiVersions_Request> is a reference to the hash representing the structure
+of the APIVERSIONS Request. it contains CorrelationId, ClientId (can be empty
+string), and ApiVersion (must be 0)
+
+=back
+
+=cut
+
+$IMPLEMENTED_APIVERSIONS->{$APIKEY_APIVERSIONS} = 0;
+sub encode_api_versions_request {
+    my ( $ApiVersions_Request ) = @_;
+
+    my @data;
+    my $request = {
+                                                # template    => '...',
+                                                # len         => ...,
+        data        => \@data,
+    };
+
+    _encode_request_header( $request, $APIKEY_APIVERSIONS, $ApiVersions_Request );
+                                                                            # Size
+                                                                            # ApiKey
+                                                                            # ApiVersion
+                                                                            # CorrelationId
+                                                                            # ClientId
+
+    return pack( $request->{template}, $request->{len}, @data );
+}
+
+my $_decode_api_version_response_template = q{x[l]l>s>l>X[l]l>/(s>s>s>)};
+                                           # x[l]    # Size (skip)
+                                           # l>      # CorrelationId
+                                           # s>      # ErrorCode
+                                           # l>      # ApiVersions array size
+                                           # X[l]
+                                           # l>/(    # ApiVersions array
+                                           #     s>     # ApiKey
+                                           #     s>     # MinVersion
+                                           #     s>     # MaxVersion
+                                           # )
+
+=head3 C<decode_api_versions_response( $bin_stream_ref )>
+
+Decodes the argument and returns a reference to the hash representing
+the structure of the APIVERSIONS Response.
+
+This function take argument. The following argument is currently recognized:
+
+=over 3
+
+=item C<$bin_stream_ref>
+
+C<$bin_stream_ref> is a reference to the encoded Response buffer. The buffer
+must be a non-empty binary string.
+
+=back
+
+=cut
+
+sub decode_api_versions_response {
+    my ( $bin_stream_ref ) = @_;
+
+    my @data = unpack( $_decode_api_version_response_template, $$bin_stream_ref );
+
+    my $i = 0;
+    my $ApiVersions_Response = {};
+
+    $ApiVersions_Response->{CorrelationId}                        =  $data[ $i++ ];   # CorrelationId
+    $ApiVersions_Response->{ErrorCode}                            =  $data[ $i++ ];   # ErrorCode
+
+    my $ApiVersions_array = $ApiVersions_Response->{ApiVersions}  =  [];
+    my $ApiVersions_array_size                                    =  $data[ $i++ ];   # ApiVersions array size
+    while ( $ApiVersions_array_size-- ) {
+        push( @$ApiVersions_array, {
+            ApiKey                                                => $data[ $i++ ],   # ApiKey
+            MinVersion                                            => $data[ $i++ ],   # MinVersion
+            MaxVersion                                            => $data[ $i++ ],   # MaxVersion
+            }
+        );
+    }
+
+    return $ApiVersions_Response;
+}
+
 # PRODUCE Request --------------------------------------------------------------
 
 =head3 C<encode_produce_request( $Produce_Request, $compression_codec )>
@@ -469,7 +609,7 @@ The following functions are available for C<Kafka::MockProtocol> module.
 Encodes the argument and returns a reference to the encoded binary string
 representing a Request buffer.
 
-This function take argument. The following argument is currently recognized:
+This function take arguments. The following arguments are currently recognized:
 
 =over 3
 
@@ -493,6 +633,8 @@ L<$COMPRESSION_SNAPPY|Kafka/$COMPRESSION_SNAPPY>.
 =back
 
 =cut
+
+$IMPLEMENTED_APIVERSIONS->{$APIKEY_PRODUCE} = 2;
 sub encode_produce_request {
     my ( $Produce_Request, $compression_codec ) = @_;
 
@@ -560,6 +702,45 @@ my $_decode_produce_response_template = qq{x[l]l>l>X[l]l>/(s>/al>X[l]l>/(l>s>${_
                                         #     )
                                         # )
 
+my $_decode_produce_response_template_v1 = qq{x[l]l>l>X[l]l>/(s>/al>X[l]l>/(l>s>${_int64_template}))l>};
+                                        # x[l]                    # Size (skip)
+                                        # l>                      # CorrelationId
+
+                                        # l>                      # topics array size
+                                        # X[l]
+                                        # l>/(                    # topics array
+                                        #     s>/a                    # TopicName
+
+                                        #     l>                      # partitions array size
+                                        #     X[l]
+                                        #     l>/(                    # partitions array
+                                        #         l>                      # Partition
+                                        #         s>                      # ErrorCode
+                                        #         $_int64_template        # Offset
+                                        #     )
+                                        # )
+                                        # l>                      # Throttle_Time_Ms
+
+my $_decode_produce_response_template_v2 = qq{x[l]l>l>X[l]l>/(s>/al>X[l]l>/(l>s>${_int64_template}${_int64_template}))l>};
+                                        # x[l]                    # Size (skip)
+                                        # l>                      # CorrelationId
+
+                                        # l>                      # topics array size
+                                        # X[l]
+                                        # l>/(                    # topics array
+                                        #     s>/a                    # TopicName
+
+                                        #     l>                      # partitions array size
+                                        #     X[l]
+                                        #     l>/(                    # partitions array
+                                        #         l>                      # Partition
+                                        #         s>                      # ErrorCode
+                                        #         $_int64_template        # Offset
+                                        #         $_int64_template        # Log_Append_Time
+                                        #     )
+                                        # )
+                                        # l>                      # Throttle_Time_Ms
+
 =head3 C<decode_produce_response( $bin_stream_ref )>
 
 Decodes the argument and returns a reference to the hash representing
@@ -578,9 +759,15 @@ must be a non-empty binary string.
 
 =cut
 sub decode_produce_response {
-    my ( $bin_stream_ref ) = @_;
+    my ( $bin_stream_ref, $api_version ) = @_;
 
-    my @data = unpack( $_decode_produce_response_template, $$bin_stream_ref );
+    $api_version //= $DEFAULT_APIVERSION;
+    my $is_v1 = $api_version == 1;
+    my $is_v2 = $api_version == 2;
+
+    my @data = unpack(   $is_v1 ? $_decode_produce_response_template_v1
+                       : $is_v2 ? $_decode_produce_response_template_v2
+                       :          $_decode_produce_response_template, $$bin_stream_ref );
 
     my $i = 0;
     my $Produce_Response = {};
@@ -598,9 +785,10 @@ sub decode_produce_response {
         my $partitions_array_size                   =  $data[ $i++ ];   # partitions array size
         while ( $partitions_array_size-- ) {
             my $partition = {
-                Partition                           => $data[ $i++ ],   # Partition
-                ErrorCode                           => $data[ $i++ ],   # ErrorCode
-                Offset                   => _unpack64( $data[ $i++ ] ), # Offset
+                Partition                           => $data[ $i++ ],          # Partition
+                ErrorCode                           => $data[ $i++ ],          # ErrorCode
+                Offset                   => _unpack64( $data[ $i++ ] ),        # Offset
+     ( $is_v2 ? (Log_Append_Time         => _unpack64( $data[ $i++ ] )) : () ), # Log_Append_Time
             };
 
             push( @$partitions_array, $partition );
@@ -608,6 +796,8 @@ sub decode_produce_response {
 
         push( @$topics_array, $topic );
     }
+    defined $data[ $i ]
+      and $Produce_Response->{Throttle_Time_Ms} = $data[ $i++ ];
 
     return $Produce_Response;
 }
@@ -631,6 +821,9 @@ the structure of the FETCH Request (examples see C<t/*_decode_encode.t>).
 =back
 
 =cut
+
+# version 0, 1, 2 have the same request protocol.
+$IMPLEMENTED_APIVERSIONS->{$APIKEY_FETCH} = 3;
 sub encode_fetch_request {
     my ( $Fetch_Request ) = @_;
 
@@ -640,23 +833,29 @@ sub encode_fetch_request {
                                                 # len         => ...,
         data        => \@data,
     };
-
-    _encode_request_header( $request, $APIKEY_FETCH, $Fetch_Request );
+    my $api_version = _encode_request_header( $request, $APIKEY_FETCH, $Fetch_Request );
                                                                             # Size
                                                                             # ApiKey
                                                                             # ApiVersion
                                                                             # CorrelationId
                                                                             # ClientId
+    my $is_v3 = $api_version == 3;
 
     push( @data, $CONSUMERS_REPLICAID );                                    # ReplicaId
     my $topics_array = $Fetch_Request->{topics};
     push( @data,
         $Fetch_Request->{MaxWaitTime},                                      # MaxWaitTime
         $Fetch_Request->{MinBytes},                                         # MinBytes
+        ( $is_v3 ? $Fetch_Request->{MaxBytes} : () ),                       # MaxBytes (version 3 only)
         scalar( @$topics_array ),                                           # topics array size
     );
-    $request->{template}    .= $_FetchRequest_header_template;
-    $request->{len}         += $_FetchRequest_header_length;
+    if ($is_v3) {
+        $request->{template}    .= $_FetchRequest_header_template_v3;
+        $request->{len}         += $_FetchRequest_header_length_v3;
+    } else {
+        $request->{template}    .= $_FetchRequest_header_template;
+        $request->{len}         += $_FetchRequest_header_length;
+    }
 
     foreach my $topic ( @$topics_array ) {
         $request->{template}    .= q{s>};                                   # string length
@@ -701,7 +900,13 @@ must be a non-empty binary string.
 
 =cut
 sub decode_fetch_response {
-    my ( $bin_stream_ref ) = @_;
+    my ( $bin_stream_ref, $api_version ) = @_;
+
+    $api_version //= $DEFAULT_APIVERSION;
+    my $is_v1 = $api_version == 1;
+    my $is_v2 = $api_version == 2;
+    my $is_v3 = $api_version == 3;
+
 
 # According to Apache Kafka documentation:
 # As an optimization the server is allowed to return a partial message at the end of the message set.
@@ -716,13 +921,16 @@ sub decode_fetch_response {
         bin_stream  => $bin_stream_ref,
     };
 
-    _decode_fetch_response_template( $response );
+    _decode_fetch_response_template( $response, $api_version );
     @data = unpack( $response->{template}, $$bin_stream_ref );
 
     my $i = 0;
     my $Fetch_Response = {};
 
     $Fetch_Response->{CorrelationId}                        =  $data[ $i++ ];   # CorrelationId
+    if ($is_v1 || $is_v2 || $is_v3) {
+        $Fetch_Response->{ThrottleTimeMs}                   =  $data[ $i++ ];   # ( ThrottleTimeMs ) only v1 and above
+    }
 
     my $topics_array = $Fetch_Response->{topics}            =  [];
     my $topics_array_size                                   =  $data[ $i++ ];   # topics array size
@@ -774,6 +982,7 @@ the structure of the OFFSET Request (examples see C<t/*_decode_encode.t>).
 =back
 
 =cut
+$IMPLEMENTED_APIVERSIONS->{$APIKEY_OFFSET} = 1;
 sub encode_offset_request {
     my ( $Offset_Request ) = @_;
 
@@ -783,14 +992,15 @@ sub encode_offset_request {
                                                 # len         => ...,
         data        => \@data,
     };
-
-    _encode_request_header( $request, $APIKEY_OFFSET, $Offset_Request );
+    my $api_version = _encode_request_header( $request, $APIKEY_OFFSET, $Offset_Request );
                                                                             # Size
                                                                             # ApiKey
                                                                             # ApiVersion
                                                                             # CorrelationId
                                                                             # ClientId
 
+    my $is_v1 = $api_version == 1;
+
     my $topics_array = $Offset_Request->{topics};
     push( @data,
         $CONSUMERS_REPLICAID,                                               # ReplicaId
@@ -799,6 +1009,8 @@ sub encode_offset_request {
     $request->{template}    .= $_OffsetRequest_header_template;
     $request->{len}         += $_OffsetRequest_header_length;
 
+    my $body_template = $is_v1 ? $_OffsetRequest_body_template_v1 : $_OffsetRequest_body_template;
+    my $body_length   = $is_v1 ? $_OffsetRequest_body_length_v1   : $_OffsetRequest_body_length;
     foreach my $topic ( @$topics_array ) {
         $request->{template}    .= q{s>};                                   # string length
         $request->{len}          += 2;
@@ -812,13 +1024,15 @@ sub encode_offset_request {
             push( @data,
                 $partition->{Partition},                                    # Partition
                 _pack64( $partition->{Time} ),                              # Time
-                $partition->{MaxNumberOfOffsets},                           # MaxNumberOfOffsets
+                $is_v1 ? () : $partition->{MaxNumberOfOffsets},             # MaxNumberOfOffsets
             );
-            $request->{template}    .= $_OffsetRequest_body_template;
-            $request->{len}         += $_OffsetRequest_body_length;
+            $request->{template}    .= $body_template;
+            $request->{len}         += $body_length;
         }
     }
 
+    # say STDERR $request->{template};
+    # say STDERR HexDump($_) foreach @data; use Data::HexDump;
     return pack( $request->{template}, $request->{len}, @data );
 }
 
@@ -847,6 +1061,25 @@ my $_decode_offset_response_template = qq{x[l]l>l>X[l]l>/(s>/al>X[l]l>/(l>s>l>X[
                                         #     )
                                         # )
 
+my $_decode_offset_response_template_v1 = qq{x[l]l>l>X[l]l>/(s>/al>X[l]l>/(l>s>${_int64_template}${_int64_template}))};
+                                        # x[l]                    # Size (skip)
+                                        # l>                      # CorrelationId
+
+                                        # l>                      # topics array size
+                                        # X[l]
+                                        # l>/(                    # topics array
+                                        #     s>/a                    # TopicName
+
+                                        #     l>                      # PartitionOffsets array size
+                                        #     X[l]
+                                        #     l>/(                    # PartitionOffsets array
+                                        #         l>                      # Partition
+                                        #         s>                      # ErrorCode
+                                        #         $_int64_template        # Timestamp
+                                        #         $_int64_template        # Offset
+                                        #     )
+                                        # )
+
 =head3 C<decode_offset_response( $bin_stream_ref )>
 
 Decodes the argument and returns a reference to the hash representing
@@ -865,12 +1098,16 @@ must be a non-empty binary string.
 
 =cut
 sub decode_offset_response {
-    my ( $bin_stream_ref ) = @_;
+    my ( $bin_stream_ref, $api_version ) = @_;
 
-    my @data = unpack( $_decode_offset_response_template, $$bin_stream_ref );
+    $api_version //= $DEFAULT_APIVERSION;
+    my $is_v1 = $api_version == 1;
+    my $template = $is_v1 ? $_decode_offset_response_template_v1 : $_decode_offset_response_template;
+
+    my @data = unpack( $template, $$bin_stream_ref );
 
     my $i = 0;
-    my $Offset_Response = {};
+    my $Offset_Response = { };
 
     $Offset_Response->{CorrelationId}                           =  $data[ $i++ ];   # CorrelationId
 
@@ -889,11 +1126,16 @@ sub decode_offset_response {
                 Partition                                       => $data[ $i++ ],   # Partition
                 ErrorCode                                       => $data[ $i++ ],   # ErrorCode
             };
+            if ($is_v1) {
+                $PartitionOffset->{Timestamp}           = _unpack64($data[ $i++ ]); # Timestamp (v1 only)
+                $PartitionOffset->{Offset}              = _unpack64($data[ $i++ ]); # Offset (v1 only)
 
-            $Offset_array = $PartitionOffset->{Offset}          =  [];
-            $Offset_array_size                                  =  $data[ $i++ ];   # Offset array size
-            while ( $Offset_array_size-- ) {
-                push( @$Offset_array,                   _unpack64( $data[ $i++ ] ) );   # Offset
+            } else {
+                $Offset_array = $PartitionOffset->{Offset}      =  [];
+                $Offset_array_size                              =  $data[ $i++ ];   # Offset array size
+                while ( $Offset_array_size-- ) {
+                    push( @$Offset_array,                   _unpack64( $data[ $i++ ] ) );   # Offset
+                }
             }
 
             push( @$PartitionOffsets_array, $PartitionOffset );
@@ -924,6 +1166,7 @@ the structure of the METADATA Request (examples see C<t/*_decode_encode.t>).
 =back
 
 =cut
+$IMPLEMENTED_APIVERSIONS->{$APIKEY_METADATA} = 0;
 sub encode_metadata_request {
     my ( $Metadata_Request ) = @_;
 
@@ -1078,30 +1321,48 @@ sub decode_metadata_response {
 sub _encode_request_header {
     my ( $request, $api_key, $request_ref ) = @_;
 
+    # we need to find out which API version to use for the request (and the
+    # response). $request_ref->{ApiVersion} can be specified by the end user,
+    # or providede by by Kafka::Connection ( see
+    # Kafka::Connection::_get_api_versions() ). If not provided, we
+    # default to $DEFAULT_APIVERSION
+    my $api_version = $request_ref->{ApiVersion} // $DEFAULT_APIVERSION;
     @{ $request->{data} } = (
                                                                             # Size
         $api_key,                                                           # ApiKey
-        $APIVERSION,                                                        # ApiVersion
+        $api_version,                                                       # ApiVersion
         $request_ref->{CorrelationId},                                      # CorrelationId
     );
     $request->{template}    = $_Request_header_template;
     $request->{len}         = $_Request_header_length;
     _encode_string( $request, $request_ref->{ClientId} );                   # ClientId
 
-    return;
+    return $api_version;
 }
 
 # Generates a template to decrypt the fetch response body
 sub _decode_fetch_response_template {
-    my ( $response ) = @_;
+    my ( $response, $api_version ) = @_;
 
-    $response->{template}       = $_FetchResponse_header_template;
-    $response->{stream_offset}  = $_FetchResponse_header_length;    # bytes before topics array size
+    my $is_v1 = $api_version == 1;
+    my $is_v2 = $api_version == 2;
+    my $is_v3 = $api_version == 3;
+
+    if ($is_v1 || $is_v2 || $is_v3) {
+        $response->{template}       = $_FetchResponse_header_template_v1;
+        $response->{stream_offset}  = $_FetchResponse_header_length_v1;    # bytes before topics array size
                                                                                 # [l] Size
                                                                                 # [l] CorrelationId
+                                                                                # [l] throttle_time_ms
+    } else {
+        $response->{template}       = $_FetchResponse_header_template;
+        $response->{stream_offset}  = $_FetchResponse_header_length;    # bytes before topics array size
+                                                                                # [l] Size
+                                                                                # [l] CorrelationId
+    }
     my $topics_array_size = unpack(
-         q{x[}.$response->{stream_offset}
-        .q{]l>},                            # topics array size
+         q{x[}.$response->{stream_offset} . q{]}
+        .q{l>},                            # topics array size
         ${ $response->{bin_stream} }
     );
     $response->{stream_offset} += 4;        # bytes before TopicName length
@@ -1143,7 +1404,7 @@ sub _decode_fetch_response_template {
 
 # Decrypts MessageSet
 sub _decode_MessageSet_array {
-    my ( $response, $MessageSetSize, $i_ref, $MessageSet_array_ref ) = @_;
+    my ( $response, $MessageSetSize, $i_ref, $MessageSet_array_ref) = @_;
 
     my $data = $response->{data};
     my $data_array_size = scalar @{ $data };
@@ -1165,7 +1426,11 @@ sub _decode_MessageSet_array {
 # A message set is also the unit of compression in Kafka,
 # and we allow messages to recursively contain compressed message sets to allow batch compression.
         $Message->{MagicByte}                                        =  $data->[ $$i_ref++ ];   # MagicByte
+        my $is_v1_msg_format = $Message->{MagicByte} == 1;
         $Message->{Attributes}                                       =  $data->[ $$i_ref++ ];   # Attributes
+        if ($is_v1_msg_format) {
+            $Message->{Timestamp}                                    =  $data->[ $$i_ref++ ];   # Timestamp
+        }
 
         $Key_length                                                  =  $data->[ $$i_ref++ ];   # Key length
         $Message->{Key}   = $Key_length   == $NULL_BYTES_LENGTH ? q{} : $data->[ $$i_ref++ ];   # Key
@@ -1175,8 +1440,11 @@ sub _decode_MessageSet_array {
         if ( my $compression_codec = $Message->{Attributes} & $COMPRESSION_CODEC_MASK ) {
             my $decompressed;
             if ( $compression_codec == $COMPRESSION_GZIP ) {
-                gunzip( \$Message->{Value} => \$decompressed )
-                    or _error( $ERROR_COMPRESSION, format_message( 'gunzip failed: %s', $GunzipError ) );
+                try {
+                    $decompressed = gunzip( $Message->{Value} );
+                } catch {
+                    _error( $ERROR_COMPRESSION, format_message( 'gunzip failed: %s', $_ ) );
+                };
             } elsif ( $compression_codec == $COMPRESSION_SNAPPY ) {
                 my ( $header, $x_version, $x_compatversion, undef ) = unpack( q{a[8]L>L>L>}, $Message->{Value} );   # undef - $x_length
 
@@ -1196,6 +1464,7 @@ sub _decode_MessageSet_array {
             } else {
                 _error( $ERROR_COMPRESSION, "Unknown compression codec $compression_codec" );
             }
+            _error( $ERROR_COMPRESSION, 'Decompression produced empty result' ) unless defined $decompressed;
             my @data;
             my $Value_length = length $decompressed;
             my $resp = {
@@ -1269,9 +1538,8 @@ sub _encode_MessageSet_array {
 
         # Compression
         if ( $compression_codec == $COMPRESSION_GZIP ) {
-            $MessageSet_array_ref->[0]->{Value} = q{};
-            gzip( \$message_set => \$MessageSet_array_ref->[0]->{Value} )
-                or _error( $ERROR_COMPRESSION, "gzip failed: $GzipError" );
+            $MessageSet_array_ref->[0]->{Value} = gzip( $message_set )
+                // _error( $ERROR_COMPRESSION, 'Unable to compress gzip data' );
         } elsif ( $compression_codec == $COMPRESSION_SNAPPY ) {
             $MessageSet_array_ref->[0]->{Value} = Compress::Snappy::compress( $message_set )
                 // _error( $ERROR_COMPRESSION, 'Unable to compress snappy data' );
@@ -1367,44 +1635,38 @@ sub _decode_MessageSet_sized_template {
 
     my $bin_stream_length = length ${ $response->{bin_stream} };
 
-    my ( $local_template, $MessageSize, $Key_length, $Value_length );
+    my ( $local_template, $MessageSize, $MagicByte, $Key_length, $Value_length );
     CREATE_TEMPLATE:
     while ( $MessageSetSize ) {
 # Not the full MessageSet
-        last CREATE_TEMPLATE if $MessageSetSize < 22;
+        last CREATE_TEMPLATE if $MessageSetSize < 22; # 22 is the minimal size of a v0 message format
                 # [q] Offset
                 # [l] MessageSize
                 # [l] Crc
                 # [c] MagicByte
                 # [c] Attributes
+                # ( [q] Timestamp ) message format v1 only
                 # [l] Key length
                 # [l] Value length
 
         $local_template = q{};
         MESSAGE_SET:
         {
-            $local_template .= $_Message_template;
-            $response->{stream_offset} += $_Message_length; # (Only Offset length)
-                # [q] Offset
-                # [l] MessageSize
-                # [l] Crc
-                # [c] MagicByte
-                # [c] Attributes
-                # [l] Key length
-                                                # bytes before MessageSize
-                                                                                # [q] Offset
-            $MessageSize = unpack(
-                 q{x[}.$response->{stream_offset}
-                .q{]l>},                        # MessageSize
+            $response->{stream_offset} += 8; # the size of the offset data value
+            ($MessageSize, $MagicByte) = unpack(
+                 q{x[}.$response->{stream_offset}.q{]}
+                .q{l>}                        # MessageSize
+                .q{x[l]}                      # Crc
+                .q{c},                        # MagicByte
+
                 ${ $response->{bin_stream} }
             );
 
-            $response->{stream_offset} += 10;   # bytes before Crc
-                                                                                # [l] MessageSize
-                                                # bytes before Key length
-                                                                                # [l] Crc
-                                                                                # [c] MagicByte
-                                                                                # [c] Attributes
+            my $is_v1_msg_format = $MagicByte == 1;
+            $local_template .= ( $is_v1_msg_format ? $_Message_template_with_timestamp : $_Message_template );
+
+            $response->{stream_offset} += ($is_v1_msg_format ? 18: 10);
+
             $Key_length = unpack(
                  q{x[}.$response->{stream_offset}
                 .q{]l>},                        # Key length
@@ -1547,9 +1809,11 @@ Sergiy Zuban
 
 Vlad Marchenko
 
+Damien Krotkine
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2016 by TrackingSoft LLC.
+Copyright (C) 2012-2017 by TrackingSoft LLC.
 
 This package is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself. See I<perlartistic> at
